// shows.c
//
// 010 ; multiple 12 channel Dodeca boards
// 011; made the shows into a subroutine which can display any routine to any channel
// 012; added program FLOWY which smoothly moves colors from one light to teh next
// 013; mod all colors to -200 - 1023
// 014; added ARCHS
// 015: removed globals - they are now in lamp[start]
// 016: changed to pointers instead of slow indexes which have to be recomputed every use
// 017: Add combineFlag to showsetup; you can combine universes to all be the same program
// 018: removed all universes under 10, eliminating old dodeca system
// 019:  Adding a byte to signal the routine continues through to the next universe also
// 020:  Initshow now automatically sets the start channels based on previous numchannels
// 021:  Add timers to show time needed per routine
// 022:  Add Tree universes 21,22,23

// universe 10 - 100 icycles
// universe 12 - 120 pixels
// universe 13 - 150 in three arches
// universe 15 - 8 in four snowmen, 10 spare and 32 off
// universe 17 - 100 on roof
// universe 14 - 100 in yard outline
// REMOVE Universe 16 -- no more windowpanes
// universe 21, 22, 23 are TREE


#include "predef.h"
#include <constants.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include <basictypes.h>
//#include <bsp.h>
#include <smarttrap.h>          // NetBurner Smart Trap utility
#include <utils.h>              // Include for LED writes on carrier board
#include <constants.h>          // Include for constands like MAIN_PRIO
#include <system.h>             // Include for system functions
#include <string.h>
#include "shows.h"
#include "xmas2011.h"
//#include <ucos.h>
#include <nbrtos.h>
#include <init.h>
//#include <cfinter.h>  // INTERRUPTS
#include <rtc.h>
#include <sim.h>
#include <pins.h>
#include <StopWatch.h>
#include "E682.h"
#include "sdcard.h"

extern int ForceGroup, ForceProg;
extern int NoFlashyFlag;
//#define deg2rad 57.29582 // degrees to radians

// PATTERN Defines
#define TEST (0)
// start of patterns

#define RGB (1)
#define COLORSTROBE (2)
#define RANDCOLORSTROBE (3)
#define TREMBLE (4)
#define COLORBLAZE (5)
#define TRADITIONAL (6)
#define RANDMORPH (7)
#define SPARKLE (8)
#define RANDSPARKLE (9)
#define FLASHDOWN (10)
#define SEQFLASHDOWN (11)
#define OFF (12)
#define GLOBALSTEADY (13)
#define GLOBALMUTATE (14)
#define RANDCOLORTOGGLE (15)
#define FASTRANDCOLORTOGGLE (16)
#define SPINFLASH (17)
#define SPINRANDFLASH (18)
#define CANDLE (19)
#define FLAME (20)
#define ICEFLAME (21)
#define MUTATE (22)
#define MUTATE2 (23)
#define FASTBLUESTROBE (24)
#define FASTGLOBALSTROBE (25)
#define BLASTER (26)
#define EXTREMEBLINK (27)
#define FASTGREENSTROBE (28)
#define BLASTER2 (29)
#define SPOOKYFLICKER (30)
#define SPOOKYWHITEFLICKER (31)
#define STROBEA (32)
#define INSANE (33)
#define SOLIDGREEN (34)
#define OOZE (35)
#define REDFLAME (36)
#define REDGREENSPOOKY (37)
#define BLUESPOOKY (38)
#define FLOWY (39)
#define GREENPULSE (40)
#define MUTATE3 (41)
#define ALTERNATOR (42)
#define BLUEPULSE (43)
#define REDGREENCHASE (44)
#define ALTREDWHITE (45)
#define STEADYWARM (46)
#define PINGPONG (47)
#define YELLOWBLENDCHASE (48)
#define NADA (49)

#define STICKSPIN (57)
#define STICKUP (51)
#define STICKUPFADE (52)
#define STICKDOWNFADE (53)
#define STICKTHRUMDOWN (54)
#define STICKTHRUMDOWNRAND (55)
#define STICKSPINFADE (56)

#define ARCHTEST (71)
#define ARCHSINGLEFADE (72)
#define ARCHMULTIBACK (73)
#define ARCHFIREFLY (74)
#define ARCHREDGRN1 (75)
#define ARCHMULTIBACKVARY (76)
#define ARCHREDGRN2 (77)
#define ARCHOFF (78)
#define ARCHCOLORFIREFLYFADE (79)
#define ARCHRAIN (80)
#define ARCHPSYCHO (81)
#define ARCHFIREFLY2 (82)

#define TREE (90)


// end of patterns

#define demo FALSE
#define test FALSE

INT16 colorsum;

// RAINBOW colors
const INT16 RAINBOWRED[] =
{ 0x00ff, 0x00ff, 0x00ff, 0x0000, 0x0000, 0x004b, 0x00ee, 0x0000, 0x00ff };
const INT16 RAINBOWGRN[] =
{ 0x0000, 0x00a5, 0x00ff, 0x0080, 0x0000, 0x0000, 0x0082, 0x0000, 0x0000 };
const INT16 RAINBOWBLU[] =
{ 0x0000, 0x0000, 0x0000, 0x0000, 0x00ff, 0x0082, 0x00ee, 0x0000, 0x0000 };

/*  SHOWSETUP ; array of data for each group of lights with data as:

 boardtype = 0 ,1 rgb, 2 onoff, 3 dual, 4 trio, 10+ ethernetUniverse
 timeinshow, nextshow, randtime
 startchannel, numchannels, COMBINE WITH NEXT GROUP
 programbegin, programend, progsel, newflag, tempflag
 stopwatch

 */
struct showdesc showsetup[]=
{
10, // Ethernet universe 10 for 100 icycles on E6804
0,400,800,
0,100,0,
RGB, YELLOWBLENDCHASE + 1, STEADYWARM, 1,0,
0,

11, // Ethernet universe 11 for 6 rgb trees on DMX; 6of9 pixels on the DMX27 driver
0,400,800,
100,6,0,
RGB, STEADYWARM + 1, RGB, 1,0,
0,

11, // Ethernet universe 11 for DMX27
0,400,800,
106,3,0,
OFF, OFF + 1, OFF, 1,0,
0,

15, // Ethernet universe 15 for snowmen
0,400,800,
109,8,0,
RGB, STEADYWARM + 1, RGB, 1,0,
0,

15, // Ethernet universe 15 for 42/50 spare
0,400,800,
117,42,0,
OFF, OFF + 1, OFF, 1,0,
0,

17, // Ethernet universe 17 for 100 pixels roof
0,400,800,
159,100,0,
RGB, STEADYWARM + 1, STEADYWARM, 1,0,
//NADA, NADA+1, NADA, 1, 0,
0,

12, // Ethernet universe 12 sticks  for 120 pixels ws2811
0,400,800,
259,120,0,
RGB, STEADYWARM + 1, RGB, 1,0,
//NADA, NADA+1, NADA, 1, 0,
0,

13, // Ethernet universe 13 for arches
0,400,800,
379,150,0,
//ARCHSINGLEFADE, ARCHZOOM2 + 1, ARCHTEST, 1,0,
ARCHSINGLEFADE, ARCHFIREFLY2 + 1, ARCHFIREFLY2, 1, 0, // 150 pixels
0,

14, // Ethernet universe 14 for yard outline
0,400,800,
529,100,0,
TREMBLE, YELLOWBLENDCHASE + 1, RGB, 1, 0, // 100 pixels
0,

// TREE, three universes for 400 pixels
21, //
0,400,2400,
629,170,1, // COMBINE
TREE, TREE + 1, RGB, 1, 0, // 170 pixels
0,

22, //
0,400,2400,
799,170,1, // COMBINE
TREE, TREE + 1, RGB, 1, 0, // 170 pixels
0,

23, //
0,400,2400,
969,60,0,
TREE, TREE + 1, RGB, 1, 0, // 170 pixels
0,

// REMEMBER to change NUMETHCHANNELS if you change the number on any ethernet universe
0,
0,0,0,
0,0,0,
0,0,0,0,0,// all zeros is the end marker
0,
};

struct desclamp * lamptr;
struct desclamp * lamptr2; // pointer to lamp

struct desclamp lamp[NUMETHLIGHTS];  // 26 bytes per lamp
struct descarch arch[NUMARCHS * 4]; // 50 lights in 3 arches
struct onoff triac[NUMONOFF]; // 32 onoff controllers

INT16 tempINT161, tempINT162, tempINT163, tempRed, tempGrn, tempBlu;
INT16 globalRed, globalGrn, globalBlu; // for combined operations
float tempfloat1, tempfloat2;
char tempchar1;
int i, j, upper, lower, pixcount;
int archi;
uint16_t reRoute;
char SleepCommand = 0;
StopWatch myStopWatch;

//volatile uint32_t timeout2;
//struct tm rtctime2;

void AllLightsOff()
    {
    SleepCommand = 1;
    }

int random(int maxout)
{
return rand()%maxout;
}

// may 22 2011 -- arch is a firefly that moves + index when 0-2, and -index when 3-5

INT16 setarch(int startpoint, int archnum, int lightnum, int setred, int setgrn, int setblu, int numberpix)
{
INT16 setarchindex;
// setarch(start, 0, 0, arch[0].R1, arch[0].G1, arch[0].B1, arch[0].ltime1);
if (archnum<0)
	return 0;
if (archnum>=NUMARCHS*2)
	return 0;
if (lightnum>PIXELSARCH)
	return 0;
if (numberpix < 1)
	return 0;
if (numberpix > PIXELSARCH)
	return 0;
// there are 50 lights in an arch
if (archnum >= (NUMARCHS)) // go the opposite direction
	{
	setarchindex = startpoint + ((archnum - NUMARCHS)*PIXELSARCH) + ((PIXELSARCH-1)-lightnum);
	lower = startpoint + ((archnum - NUMARCHS)*PIXELSARCH);
	upper = lower + PIXELSARCH;
	j=-1;
	}
else
    {
	setarchindex = startpoint + (archnum * PIXELSARCH) + lightnum;
	lower = startpoint + (archnum*PIXELSARCH);
	upper = lower + PIXELSARCH;
	j=1;
    }
for (archi=0; archi<numberpix; archi++)
    {
    if ((setarchindex >= lower)&&(setarchindex < upper))
	{
	setlight(setarchindex, setred, setgrn, setblu);
	lamp[setarchindex].R1=setred;
	lamp[setarchindex].G1=setgrn;
	lamp[setarchindex].B1=setblu;
	}
    setarchindex += j;
    }
return setarchindex;
}

INT16 setarchfade(int startpoint, int archnum, int lightnum, int setred, int setgrn, int setblu, int decay, int numberpix)
{
	INT16 setarchindex;

	if (archnum<0)
		return 0;
	if (archnum>=NUMARCHS*2)
		return 0;
	if (lightnum>PIXELSARCH)
		return 0;
	if (numberpix < 1)
		return 0;
	if (numberpix >= PIXELSARCH)
		return 0;
	if (archnum >= (NUMARCHS)) // there are 50 lights in 3 archs
		{
		setarchindex = startpoint + ((archnum - NUMARCHS)*PIXELSARCH) + ((PIXELSARCH-1)-lightnum);
		lower = startpoint + ((archnum - NUMARCHS)*PIXELSARCH);
		upper = lower + PIXELSARCH;
		j=-1;
		}
	else
	    {
		setarchindex = startpoint + (archnum * PIXELSARCH) + lightnum;
		lower = startpoint + (archnum  *PIXELSARCH);
		upper = lower + PIXELSARCH;
		j=1;
	    }

	for (archi=0; archi<numberpix; archi++)
	    {
	    if ((setarchindex >= lower)&&(setarchindex < upper))
		{
		lamp[setarchindex].R1=setred;
		lamp[setarchindex].G1=setgrn;
		lamp[setarchindex].B1=setblu;
	//	lamp[setarchindex].R2=(setred - arch[0].R1)/decay;
	//	lamp[setarchindex].G2=(setgrn - arch[0].G1)/decay;
	//	lamp[setarchindex].B2=(setblu - arch[0].B1)/decay;
		lamp[setarchindex].ltime1 = decay; // number of clicks to decay
		}
	    setarchindex += j;
	    }
	return setarchindex;
}

void initshows()
{
	UINT16 Startchan = 0;
	myStopWatch.Clear();
i=0;
while (showsetup[i].type != 0)
	{
	showsetup[i].timeinshow = 0;
	showsetup[i].nextshow = 1200; // 60sec
	showsetup[i].newflag = TRUE;
	// Allow autoIncrementing of UINT16 showsetup.startchannel
	showsetup[i].startchannel = Startchan;
	Startchan += showsetup[i].numchannels;
	iprintf("channels %d to %d on %d\n\r", showsetup[i].startchannel, Startchan, i);
	i+=1;
	}

}

void newshow( UINT16 group)
{
//timeout2=rtctime2.tm_sec + (rtctime2.tm_min*60) + (rtctime2.tm_hour*3600);
//srand(timeout2);

tempINT163 = showsetup[group].progend - showsetup[group].progbegin;  // don't execute Test
tempINT162 = random(tempINT163) + showsetup[group].progbegin;
showsetup[group].progsel = tempINT162;
// CHANGE -- if global NoFlashyFlag is set, force to GLOBALSTEADY
if (NoFlashyFlag)
	showsetup[group].progsel = GLOBALSTEADY;
showsetup[group].timeinshow = 0;
showsetup[group].nextshow = random(showsetup[group].randtime) +
	showsetup[group].randtime;
showsetup[group].newflag = TRUE;

}

UINT16 SmellCurrentShow( UINT16 group)
{
if (showsetup[group].type == 0)
	return 0;
return showsetup[group].progsel;
}

double TimeCurrentShow( UINT16 group)
{
if (showsetup[group].type == 0)
	return 0;
return showsetup[group].stopwatch;
}


void doshow()
{
static INT16 ishow;
INT16 totalChans, jshow;
StopWatch myStopWatch;
unsigned long long templonglong;

ishow = 0;
while(showsetup[ishow].type != 0)
    {
	if(SleepCommand) // Order all lights to OFF
	    {
	    showsetup[ishow].timeinshow = showsetup[ishow].nextshow;
	    }
	showsetup[ishow].timeinshow += 1;
	if (showsetup[ishow].timeinshow > showsetup[ishow].nextshow)
		{
		if (SleepCommand)
		    { // and by manually setting the program to OFF
		    showsetup[ishow].timeinshow = 0;
		    showsetup[ishow].newflag = TRUE;
		    showsetup[ishow].progsel = OFF;
		    }
		else
		    {
		    newshow(ishow);
		    }
		iprintf("group %d ON %d ", ishow, showsetup[ishow].progsel);
		}
	totalChans = showsetup[ishow].numchannels;
	jshow = ishow; // starting group

	if (showsetup[ishow].type>=10) // ETHERNET UNIVERSES ONLY
		{// assign the address to the ethernet send list
		EthSetDesc((unsigned char)showsetup[ishow].type, showsetup[ishow].startchannel, showsetup[ishow].numchannels);
		}
	while (showsetup[ishow].combineflag != 0)
	    {
	    ishow += 1;
	    totalChans += showsetup[ishow].numchannels; // add next group to the same program

	    if (showsetup[ishow].type>=10) // ETHERNET UNIVERSE TO SEND TO
			{// assign the address to the ethernet send list
			EthSetDesc((unsigned char)showsetup[ishow].type, showsetup[ishow].startchannel, showsetup[ishow].numchannels);
			}
	    }
	reRoute = 0;

	// Change Dec2021  Is a FORCE GrOUP PROG command in effect?
	if (ForceProg != 0)
		{
		// is this the group in question?
		if (ForceGroup == jshow)
			{
			// copy data and force this group to go to the command program
			showsetup[jshow].progsel = ForceProg;
			showsetup[jshow].newflag = TRUE;
			showsetup[jshow].timeinshow = 0;
			ForceProg = 0;
			ForceGroup = 0;
			}
		}

	// Time the routine
	myStopWatch.Start();
	/* execute show routines for this group of channels */
	runshows(showsetup[jshow].progsel,
			showsetup[jshow].startchannel,
			totalChans,
			showsetup[jshow].newflag);
	// SEE if the program redirects to a different routine
	if (reRoute != 0) // we want to execute a different routine instead
	    {
	    showsetup[jshow].progsel = reRoute;
	    reRoute = 0;
	    runshows(showsetup[jshow].progsel,
				showsetup[jshow].startchannel,
				totalChans,
				showsetup[jshow].newflag);
	    }
	showsetup[jshow].newflag = FALSE;
	templonglong = myStopWatch.Stop(); // finish time measurement
	showsetup[jshow].stopwatch = myStopWatch.Convert(templonglong);
	ishow += 1;
	}
SleepCommand = 0;
}

void runshows(int showsel, int start, int num, char newFA)
{
// This is executed for every group every 1/20th second
switch(showsel)
	{
	case TEST:
	clearlights(start, num);
	for (i=0; i<num; i++)
		{
		setlight(i+start,1023,1023,1023);// light one is white
		}
	break;

	case RGB:
	//clearlights(start,num);
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
		    lamptr = &lamp[i+start];
			lamptr->ltime1 = 0;
			}
		iprintf("RGB\r\n");
		newFA = FALSE;
		}
	for (i=0; i<num; i++)
		{
	    lamptr = &lamp[i+start];
		lamptr->ltime1 +=1;
		if (lamptr->ltime1 >=80)
			lamptr->ltime1=0; // reset cycling lights
		setlight(i+start,0,0,0);// light IS OFF
    	if (lamptr->ltime1>20)
        		{
			setlight(i+start,1023,0,0);// light  is red
        		}
    	if (lamptr->ltime1>40)
        		{
    		setlight(i+start,0,1023,0);// light  is green
        		}
    	if (lamptr->ltime1>60)
        		{
			setlight(i+start,0,0,1023);// light  is blue
        		}
		}
	break;

	case COLORSTROBE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(5)+1;
			lamptr->T01 = tempINT161;
			tempINT161 += random(8)+2;
			lamptr->T12 = tempINT161;
			// set a color for the lamp
			lamptr->R1 = random(800) - 200;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			lamptr->G1 = random(800) - 200;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			lamptr->B1 = random(600) - 200;
			if (lamptr->B1<0)
				lamptr->B1=0;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("COLORSTROBE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			tempRed = lamptr->R1;
			tempGrn = lamptr->G1;
			tempBlu = lamptr->B1;
			setlight(i+start,tempRed,tempGrn,tempBlu);
			}

		if (lamptr->ltime1 < lamptr->T12)
			lamptr->ltime1 += 1;
		else
			lamptr->ltime1 = 0;
		}
	break;

	case RANDCOLORSTROBE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(5)+1;
			lamptr->T01 = tempINT161;
			tempINT161 += random(8)+2;
			lamptr->T12 = tempINT161;
			// set a color for the lamp
			lamptr->R1 = random(1223) - 200;
			lamptr->G1 = random(1223) - 200;
			lamptr->B1 = random(1223) - 200;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("RANDCOLORSTROBE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			tempRed = lamptr->R1;
			tempGrn = lamptr->G1;
			tempBlu = lamptr->B1;
			setlight(i+start,tempRed,tempGrn,tempBlu);
			}

		if (lamptr->ltime1 < lamptr->T12)
			lamptr->ltime1 += 1;
		else
			{
			lamptr->R1 = random(1223)-200; // black is okay here
			lamptr->G1 = random(1223)-200;
			lamptr->B1 = random(1223)-200;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			lamptr->ltime1 = 0;
			}
		}
	break;

	case TREMBLE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(16)+4;
			lamptr->T01 = tempINT161;
			tempINT161 = random(16)+4;
			lamptr->T12 = tempINT161;
			tempINT161 = random(16)+4;
			lamptr->T23 = tempINT161;
			// set a color for the lamp
			lamptr->R1 = random(1223)-200;
			lamptr->G1 = random(1223)-200;
			lamptr->B1 = random(1223)-200;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			// clear ltime1;
			lamptr->ltime1 = 0;
			lamptr->ltime2 = 0;
			lamptr->ltime3 = 0;
			}
		iprintf("TREMBLE\r\n");
		newFA = FALSE;
		}
	for (i=0; i<num; i++)
		{//RED
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			tempRed = lamptr->R1;
			lamptr->ltime1 += 1;
			}
		else
			{
			lamptr->R1 = random(1223)-200;
			if (lamptr->R1<0)
				lamptr->R1 = 0;
			tempRed = lamptr->R1;
			lamptr->ltime1 = 0;
			}
		//Green
		if (lamptr->ltime2 < lamptr->T12)
			{
			tempGrn = lamptr->G1;
			lamptr->ltime2 += 1;
			}
		else
			{
			lamptr->G1 = random(1223)-200;
			if (lamptr->G1 < 0 )
				lamptr->G1 = 0;
			tempGrn = lamptr->G1;
			lamptr->ltime2 = 0;
			}
		//Blue
		if (lamptr->ltime3 < lamptr->T23)
			{
			tempBlu = lamptr->B1;
			lamptr->ltime3 += 1;
			}
		else
			{
			lamptr->B1 = random(1223)-200;
			if (lamptr->B1 < 0)
				lamptr->B1 = 0;
			tempBlu = lamptr->B1;
			lamptr->ltime3 = 0;
			}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}
	break;

	case COLORBLAZE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(10)+5;
			lamptr->T01 = tempINT161;
			tempINT161 += random(30)+40;
			lamptr->T12 = tempINT161;
			// set a color for the lamp -- shifted 4 places for math precision
			lamptr->R1 = (random(1023))<<4;
			lamptr->R3 = lamptr->R1;
			lamptr->G1 = (random(1023))<<4;
			lamptr->G3 = lamptr->G1;
			lamptr->B1 = (random(1023))<<4;
			lamptr->B3 = lamptr->B1;
			// set the decrements for the colors over two seconds
			lamptr->R2 = lamptr->R1 / 40;
			lamptr->G2 = lamptr->G1 / 40;
			lamptr->B2 = lamptr->B1 / 40;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("COLORBLAZE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T01)
			{
			tempRed = lamptr->R1 >>4;
			tempGrn = lamptr->G1 >>4;
			tempBlu = lamptr->B1 >>4;
			}
		else // Fade the light out
			{
			if (lamptr->R1 > lamptr->R2)
				lamptr->R1 -= lamptr->R2;
			if (lamptr->G1 > lamptr->G2)
				lamptr->G1 -= lamptr->G2;
			if (lamptr->B1 > lamptr->B2)
				lamptr->B1 -= lamptr->B2;
			tempRed = lamptr->R1 >>4;
			tempGrn = lamptr->G1 >>4;
			tempBlu = lamptr->B1 >>4;
			}
		if (lamptr->ltime1 >= lamptr->T12)
			{ // restore the full intensity
			lamptr->R1 = lamptr->R3;
			lamptr->G1 = lamptr->G3;
			lamptr->B1 = lamptr->B3;
			lamptr->ltime1 = 0;
			}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}
	break;
// NOVEMBER 3 Traditional blinking independent R G and B bulbs

	case TRADITIONAL:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// times for each bulb to turn on and off
			lamptr->T01 = random(45)+15;
			lamptr->T12 = random(45)+15;
			lamptr->T23 = random(45)+15;
			// set a color for the lamp
			lamptr->R1 = 1023;
			lamptr->G1 = 1023;
			lamptr->B1 = 1023;
			// clear ltime1;
			lamptr->ltime1 = 0;
			lamptr->ltime2 = 0;
			lamptr->ltime3 = 0;
			}
		iprintf("TRADITIONAL\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempRed = 0; // clear the temps to prevent old data ruining the show
		tempGrn = 0;
		tempBlu = 0;
		if (lamptr->ltime1 < lamptr->T01)
			{
			tempRed = lamptr->R1;
			}
		if (lamptr->ltime1 < (lamptr->T01 * 2))
			lamptr->ltime1 += 1;
		else
			{
			lamptr->ltime1 = 0;
			}

		if (lamptr->ltime2 < lamptr->T12)
			{
			tempGrn = lamptr->G1;
			}
		if (lamptr->ltime2 < (lamptr->T12 * 2))
			lamptr->ltime2 += 1;
		else
			{
			lamptr->ltime2 = 0;
			}

		if (lamptr->ltime3 < lamptr->T23)
			{
			tempBlu = lamptr->B1;
			}
		if (lamptr->ltime3 < (lamptr->T23 * 2))
			lamptr->ltime3 += 1;
		else
			{
			lamptr->ltime3 = 0;
			}

		setlight(i+start,tempRed,tempGrn,tempBlu);
		}

	break;

// morph back and forth between random colors
	case RANDMORPH:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time between colors
			tempINT161 = random(100)+100; // 5-10 seconds
			lamptr->T01 = tempINT161;

			// set a color for the lamp -- shifted 5 places for math precision
			lamptr->R3 = (random(1223) -200)*32;
			lamptr->R1 = 0;
			lamptr->G3 = (random(1223) - 200)*32;
			lamptr->G1 = 0;
			lamptr->B3 = (random(1223) -200)*32;
			lamptr->B1 = 0;
			if (lamptr->R3 <0)
				lamptr->R3 = 0;
			if (lamptr->G3<0)
				lamptr->G3 = 0;
			if (lamptr->B3<0)
				lamptr->B3 = 0;
			// set the increments for the colors over the time period
			lamptr->R2 = lamptr->R3 / tempINT161;
			lamptr->G2 = lamptr->G3 / tempINT161;
			lamptr->B2 = lamptr->B3 / tempINT161;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("RANDMORPH\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			lamptr->R1+=lamptr->R2;
			tempRed = lamptr->R1 >> 5;
			lamptr->G1+=lamptr->G2;
			tempGrn = lamptr->G1 >> 5;
			lamptr->B1+=lamptr->B2;
			tempBlu = lamptr->B1 >> 5;
			lamptr->ltime1 += 1;
			}
		else // change to new color
			{
			lamptr->R3 = (random(1223) -200)*32;
			lamptr->G3 = (random(1223) -200)*32;
			lamptr->B3 = (random(1223) -200)*32;
			if (lamptr->R3 <0)
				lamptr->R3 = 0;
			if (lamptr->G3<0)
				lamptr->G3 = 0;
			if (lamptr->B3<0)
				lamptr->B3 = 0;
			tempRed = lamptr->R1 >> 5;
			tempGrn = lamptr->G1 >> 5;
			tempBlu = lamptr->B1 >> 5;
			// set T01 to time between colors
			tempINT161 = random(100)+100; // 5-20 seconds
			lamptr->T01 = tempINT161;
			// calculate the increments over the time period
			tempINT162 = lamptr->R3 - lamptr->R1;
			lamptr->R2 = tempINT162 / tempINT161;
			tempINT162 = lamptr->G3 - lamptr->G1;
			lamptr->G2 = tempINT162 / tempINT161;
			tempINT162 = lamptr->B3 - lamptr->B1;
			lamptr->B2 = tempINT162 / tempINT161;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}
	break;

	case SPARKLE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(10)+1;
			lamptr->T01 = tempINT161;
			tempINT161 += random(100)+60;
			lamptr->T12 = tempINT161;
			// set a color for the lamp
			lamptr->R1 = random(611)-100;
			lamptr->G1 = random(611)-100;
			lamptr->B1 = random(611)-100;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("SPARKLE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			tempRed = lamptr->R1 * 2;
			tempGrn = lamptr->G1 * 2;
			tempBlu = lamptr->B1 * 2;
			setlight(i+start,tempRed,tempGrn,tempBlu);
			lamptr->ltime1 += 1;
			}
		else // normal brightness
			{
			tempRed = lamptr->R1;
			tempGrn = lamptr->G1;
			tempBlu = lamptr->B1;
			setlight(i+start,tempRed,tempGrn,tempBlu);
			lamptr->ltime1 += 1;
			}
		if (lamptr->ltime1 >= lamptr->T12)
			{
			lamptr->ltime1 = 0;
			}
		}
	break;

	case RANDSPARKLE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(5)+1;
			lamptr->T01 = tempINT161;
			tempINT161 += random(30)+60;
			lamptr->T12 = tempINT161;
			// set a color for the lamp
			lamptr->R1 = random(612)-100;
			lamptr->G1 = random(612)-100;
			lamptr->B1 = random(612)-100;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("RANDSPARKLE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T01)
			{ // white flash
			tempRed = 1023;
			tempGrn = 1023;
			tempBlu = 1023;
			setlight(i+start,tempRed,tempGrn,tempBlu);
			}
		else // followed by a color
			{
			tempRed = lamptr->R1;
			tempGrn = lamptr->G1;
			tempBlu = lamptr->B1;
			setlight(i+start,tempRed,tempGrn,tempBlu);
			}
		if (lamptr->ltime1 >= lamptr->T12)
			{
			lamptr->R1 = random(612)-100;
			lamptr->G1 = random(612)-100;
			lamptr->B1 = random(612)-100;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			lamptr->ltime1 = 0;
			}
		}
	break;

	case FLASHDOWN:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			tempINT161 = random(5)+1;
			lamptr->T01 = tempINT161;
			tempINT161 += 60; // three second fade
			lamptr->T12 = tempINT161;
			tempINT161 += random(40)+10; // second wait
			lamptr->T23 = tempINT161;
			// set a color for the lamp shifted 4 over for math precision
			tempINT161 = random(1223)-200;
			if (tempINT161 <0 )
				tempINT161 = 0;
			colorsum = tempINT161;
			lamptr->R1 =tempINT161 << 4;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0 )
				tempINT161 = 0;
			colorsum += tempINT161;
			lamptr->G1 = tempINT161 << 4;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0 )
				tempINT161 = 0;
			colorsum += tempINT161;
			lamptr->B1 = tempINT161 << 4;
			if (colorsum <300)
				{
				lamptr->R1 += 700;
				lamptr->G1 += 700;
				lamptr->B1 += 700;
				}
			lamptr->R2 = (lamptr->R1) / 60;
			lamptr->G2 = (lamptr->G1) / 60;
			lamptr->B2 = (lamptr->B1) / 60;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("FLASHDOWN\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempRed = lamptr->R1>>4;
		tempGrn = lamptr->G1>>4;
		tempBlu = lamptr->B1>>4;
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T01)
			{ // white flash
			tempRed = 1023;
			tempGrn = 1023;
			tempBlu = 1023;
			}
		else if (lamptr->ltime1 < lamptr->T12)
			// followed by a fade to the color
			{
			lamptr->R1 -= lamptr->R2;
			tempRed = lamptr->R1>>4;
			lamptr->G1 -= lamptr->G2;
			tempGrn = lamptr->G1>>4;
			lamptr->B1 -= lamptr->B2;
			tempBlu = lamptr->B1>>4;
			}
		if (lamptr->ltime1 >= lamptr->T23)
			{
			// set a color for the lamp shifted 4 over for math precision
			tempINT161 = random(1223)-200;
			if (tempINT161 <0 )
				tempINT161 = 0;
			colorsum = tempINT161;
			lamptr->R1 =tempINT161 << 4;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0 )
				tempINT161 = 0;
			colorsum += tempINT161;
			lamptr->G1 = tempINT161 << 4;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0 )
				tempINT161 = 0;
			colorsum += tempINT161;
			lamptr->B1 = tempINT161 << 4;
			if (colorsum <300)
				{
				lamptr->R1 += 700;
				lamptr->G1 += 700;
				lamptr->B1 += 700;
				}
			lamptr->R2 = (lamptr->R1) / 60;
			lamptr->G2 = (lamptr->G1) / 60;
			lamptr->B2 = (lamptr->B1) / 60;
			lamptr->ltime1 = 0;
			}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}
	break;

	case SEQFLASHDOWN:
	if (newFA == TRUE)
		{
		j = 0;
		// set a color for the lamp shifted 4 over for math precision
		tempINT161 = random(1200)-200;
		if (tempINT161 <0 )
			tempINT161 = 0;
		lamp[start].R3 =tempINT161 << 4;
		tempINT161 = random(1200)-200;
		if (tempINT161 <0 )
			tempINT161 = 0;
		lamp[start].G3 = tempINT161 << 4;
		tempINT161 = random(1200)-200;
		if (tempINT161 <0 )
			tempINT161 = 0;
		lamp[start].B3 = tempINT161 << 4;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = 4;
			lamptr->T01 = tempINT161;
			tempINT161 += 50; // two plus second fade
			lamptr->T12 = tempINT161;
			tempINT161 += 36;
			lamptr->T23 = tempINT161;
			lamptr->ltime1 = j * 15; // each light is at a different phase

			j+= 1;
			if (j>=6)
				j=0;

			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			lamptr->R2 = (lamp[start].R3)/50;
			lamptr->G2 = (lamp[start].G3)/50;
			lamptr->B2 = (lamp[start].B3)/50;
			}
		iprintf("SEQFLASHDOWN\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 <= lamptr->T01)
			{ // white flash
			tempRed = 1023;
			tempGrn = 1023;
			tempBlu = 1023;
			}
		else if (lamptr->ltime1 < lamptr->T12)
			// followed by a fade to the color
			{
			lamptr->R1 -= lamptr->R2;
			tempRed = lamptr->R1>>4;
			lamptr->G1 -= lamptr->G2;
			tempGrn = lamptr->G1>>4;
			lamptr->B1 -= lamptr->B2;
			tempBlu = lamptr->B1>>4;
			}
		else
		{
			tempRed = 0;
			tempGrn = 0;
			tempBlu = 0;
		}

		if (lamptr->ltime1 > lamptr->T23)
			{
			lamptr->R1 = lamp[start].R3;
			lamptr->G1 = lamp[start].G3;
			lamptr->B1 = lamp[start].B3;
			lamptr->ltime1 = 0;
			}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}

	break;

	case OFF:
		if (newFA == TRUE)
		{
			iprintf("OFF\r\n");
			newFA = FALSE;
		}

		clearlights(start,num);
	break;

	case GLOBALSTEADY:
	if (newFA == TRUE)
		{
		globalRed = (random(1220)-200);
		if (globalRed<0)
			globalRed = 0;
		globalGrn = (random(1220)-200);
		if (globalGrn < 0)
			globalGrn = 0;
		globalBlu = (random(1220)-200);
		if (globalBlu<0)
			globalBlu = 0;
		for (i=0; i<num; i++)
			{
			setlight(i+start,globalRed,globalGrn,globalBlu);
			}
		iprintf("GLOBALSTEADY\r\n");
		newFA = FALSE;
		}
	break;

	case GLOBALMUTATE:
		if (newFA == TRUE)
			{
			lamp[start].R3 = (random(350)-100)*4;
			lamp[start].G3 = (random(350)-100)*4;
			lamp[start].B3 = (random(350)-100)*4;
			tempINT161 = 0;
			tempINT162 = 0;
			tempINT163 = 0;
			iprintf("GLOBALMUTATE\r\n");
			newFA = FALSE;
			}
		clearlights(start,num);
		tempINT161 = lamp[start].R1;
		if (abs(tempINT161 - lamp[start].R3) <4)
			lamp[start].R3 = (random(350)-100)*4;
		if (tempINT161 < lamp[start].R3)
			tempINT161 += 4;
		if (tempINT161 > lamp[start].R3)
			tempINT161 -= 4;
		lamp[start].R1 = tempINT161;
		tempINT162 = lamp[start].G1;
		if (abs(tempINT162 - lamp[start].G3) < 4)
			lamp[start].G3 = (random(350)-100)*4;
		if (tempINT162 < lamp[start].G3)
			tempINT162 += 4;
		if (tempINT162 > lamp[start].G3)
			tempINT162 -= 4;
		lamp[start].G1 = tempINT162;
		tempINT163 = lamp[start].B1;
		if (abs(tempINT163 - lamp[start].B3) < 4)
			lamp[start].B3 = (random(350)-100)*4;
	 	if (tempINT163 < lamp[start].B3)
			tempINT163 += 4;
		if (tempINT163 > lamp[start].B3)
			tempINT163 -= 4;
		lamp[start].B1 = tempINT163;
		for (i=0; i<num; i++)
			{
			setlight(i+start,lamp[start].R1, lamp[start].G1, lamp[start].B1);
			}
		break;


	case FASTBLUESTROBE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(8)+3;
			lamptr->T01 = tempINT161;
			tempINT161 += random(16)+3;
			lamptr->T12 = tempINT161;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("FASTBLUESTROBE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			if (lamptr->B1 == 0)
				{
				lamptr->B1 = 1023;
				setlight(i+start,0,0,1023);
				}
			else
				{
				lamptr->B1 = 0;
				setlight(i+start,0,0,0);
				}
			}
		if (lamptr->ltime1 < lamptr->T12)
			lamptr->ltime1 += 1;
		else
			{
			lamptr->ltime1 = 0;
			tempINT161 = random(8)+3;
			}
		}
	break;

	case FASTGLOBALSTROBE:
	if (newFA == TRUE)
		{
		lamp[start].R3 = (random(1200)-200);
		if (lamp[start].R3<0)
			lamp[start].R3 = 0;
		lamp[start].G3 = (random(1200)-200);
		if (lamp[start].G3<0)
			lamp[start].G3 = 0;
		lamp[start].B3 = (random(1200)-200);
		if (lamp[start].B3<0)
			lamp[start].B3 = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(8)+3;
			lamptr->T01 = tempINT161;
			tempINT161 += random(16)+3;
			lamptr->T12 = tempINT161;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("FASTGLOBALSTROBE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			if (lamptr->B1 == 0)
				{
				lamptr->B1 = 1023;
				setlight(i+start,lamp[start].R3, lamp[start].G3, lamp[start].B3);
				}
			else
				{
				lamptr->B1 = 0;
				setlight(i+start,0,0,0);
				}
			}
		if (lamptr->ltime1 < lamptr->T12)
			lamptr->ltime1 += 1;
		else
			{
			lamptr->ltime1 = 0;
			tempINT161 = random(8)+3;
			}
		}
	break;

	case RANDCOLORTOGGLE:
	if (newFA == TRUE)
		{
		lamp[start].ltime1=0;
		iprintf("RANDCOLORTOGGLE\r\n");
		newFA = FALSE;
		}
	lamp[start].ltime1+=1;
	if (lamp[start].ltime1 >= 10)
		{
		tempINT161 = random(num)+start; // pick a random light
		tempRed = (random(1200)-200);
		tempGrn = (random(1200)-200);
		tempBlu = (random(1200)-200);
		if (tempRed<0)
			tempRed = 0;
		if (tempGrn<0)
			tempGrn = 0;
		if (tempBlu<0)
			tempBlu = 0;
		setlight(tempINT161, tempRed, tempGrn, tempBlu);
		lamp[start].ltime1 = 0;
		}
	break;

	case FASTRANDCOLORTOGGLE:
	if (newFA == TRUE)
		{
		lamp[start].ltime1=0;
		iprintf("FASTRANDCOLORTOGGLE\r\n");
		newFA = FALSE;
		}
		tempINT161 = random(num)+start; // pick a random light
		tempRed = (random(1200)-200);
		tempGrn = (random(1200)-200);
		tempBlu = (random(1200)-200);
		if (tempRed<0)
			tempRed = 0;
		if (tempGrn<0)
			tempGrn = 0;
		if (tempBlu<0)
			tempBlu = 0;
	setlight(tempINT161, tempRed, tempGrn, tempBlu);
	break;


	case SPINFLASH:
	if (newFA == TRUE)
		{
		lamp[start].R3 = (random(1200)-200);
		lamp[start].G3 = (random(1200)-200);
		lamp[start].B3 = (random(1200)-200);
		if (lamp[start].R3<0)
			lamp[start].R3 = 0;
		if (lamp[start].G3<0)
			lamp[start].G3 = 0;
		if (lamp[start].B3<0)
			lamp[start].B3 = 0;
		j = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = j * 5;
			j+=1;
			if (j>=12)
				j=0;
			}
		iprintf("SPINFLASH\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < 13)
			{
			setlight(i+start,lamp[start].R3, lamp[start].G3, lamp[start].B3);
			}
		if (lamptr-> ltime1 >= 60)
			lamptr->ltime1 = 0;
		else
			lamptr->ltime1 += 1;
		}
	break;

	case SPINRANDFLASH:
	if (newFA == TRUE)
		{
		lamp[start].R3 = (random(1200)-200);
		lamp[start].G3 = (random(1200)-200);
		lamp[start].B3 = (random(1200)-200);
		if (lamp[start].R3<0)
			lamp[start].R3 = 0;
		if (lamp[start].G3<0)
			lamp[start].G3 = 0;
		if (lamp[start].B3<0)
			lamp[start].B3 = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = random(612)-100;
			lamptr->G1 = random(612)-100;
			lamptr->B1 = random(612)-100;
			if (lamptr->R1 <0)
				lamptr->R1 = 0;
			if (lamptr->G1<0)
				lamptr->G1 = 0;
			if (lamptr->B1<0)
				lamptr->B1 = 0;
			lamptr->ltime1 = i * 10;
			}
		iprintf("SPINRANDFLASH\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempRed = lamp[start].R3;
		tempGrn = lamp[start].G3;
		tempBlu = lamp[start].B3;
		if (lamptr->ltime1 < 10)
			{
			tempRed = lamptr->R1;
			tempGrn = lamptr->G1;
			tempBlu = lamptr->B1;
			}
		if (lamptr->ltime1 >= 120)
			{
			lamptr->ltime1 = 0;
			}
		else
			lamptr->ltime1 += 1;
		setlight(i+start,tempRed, tempGrn, tempBlu);
		}
	break;
// November 5
	case CANDLE:
	if (newFA == TRUE)
		{
		// Orange to yellow
		lamp[start].R3 = 1023;
		lamp[start].G3 = 768;
		lamp[start].B3 = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->G1 = random(512);
			lamptr->ltime1 = random(5); // random delay on each light
			}
		iprintf("CANDLE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempRed = lamp[start].R3;
		tempGrn = lamp[start].G3 + lamptr->G1;
		tempBlu = lamp[start].B3;
		if (lamptr->ltime1 >= 2)
			{
			if (lamptr->G1 < 128) // center lights color around <1, .75, 0>
				lamptr->G1 += random(128);
			else
				lamptr->G1 -= random(128);
			lamptr->ltime1 = 0;
			}
		lamptr->ltime1 += 1;
		setlight(i+start,tempRed, tempGrn, tempBlu);
		}
	break;

	case FLAME:
	if (newFA == TRUE)
		{
		// red to yellow
		lamp[start].R3 = 1023;
		lamp[start].G3 = 0;
		lamp[start].B3 = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->G1 = random(512);
			lamptr->ltime1 = random(5); // random delay on each light
			}
		iprintf("FLAME\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempRed = lamp[start].R3;
		tempGrn = lamp[start].G3 + lamptr->G1;
		tempBlu = lamp[start].B3;
		if (lamptr->ltime1 >= 2)
			{
			if (lamptr->G1 < 512)
				lamptr->G1 += random(512);
			else
				lamptr->G1 -= random(512);
			lamptr->ltime1 = 0;
			}
		lamptr->ltime1 += 1;
		setlight(i+start,tempRed, tempGrn, tempBlu);
		}
	break;

	case ICEFLAME:
	if (newFA == TRUE)
		{
		// blue to tourquise
		lamp[start].R3 = 205;
		lamp[start].G3 = 0;
		lamp[start].B3 = 1023;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->G1 = random(1023);
			lamptr->ltime1 = random(5); // random delay on each light
			}
		iprintf("ICEFLAME\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempRed = lamp[start].R3;
		tempGrn = lamp[start].G3 + lamptr->G1;
		tempBlu = lamp[start].B3;
		if (lamptr->ltime1 >= 3)
			{
			if (lamptr->G1 <= 512)
				lamptr->G1 += random(511);
			else
				lamptr->G1 -= random(510);
			lamptr->ltime1 = 0;
			}
		lamptr->ltime1 += 1;
		setlight(i+start,tempRed, tempGrn, tempBlu);
		}
	break;

	case MUTATE:
	case MUTATE2:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			tempINT161 = random(300) - 50;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->R3 = tempINT161*4;
			tempINT161 = random(300) - 50;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->G3 = tempINT161*4;
 			tempINT161 = random(300) - 50;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->B3 = tempINT161*4;
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			}
		iprintf("MUTATE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (abs(lamptr->R1 - lamptr->R3) <4)
			{
			//lamptr->R3 = random(250)<<2;
			tempINT161 = random(300) - 50;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->R3 = tempINT161<<2;
			}
		if (lamptr->R1 < lamptr->R3)
			lamptr->R1 += 4;
		else
			lamptr->R1 -= 4;
		if (abs(lamptr->G1 - lamptr->G3) <4)
			{
			//lamptr->G3 = random(250)<<2;
			tempINT161 = random(300) - 50;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->G3 = tempINT161<<2;
			}
		if (lamptr->G1 < lamptr->G3)
			lamptr->G1 += 4;
		else
			lamptr->G1 -= 4;
		if (abs(lamptr->B1 - lamptr->B3) <4)
			{
			//lamptr->B3 = random(250)<<2;
			tempINT161 = random(300) - 50;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->B3 = tempINT161<<2;
			}
		if (lamptr->B1 < lamptr->B3)
			lamptr->B1 += 4;
		else
			lamptr->B1 -= 4;

		setlight(i+start, lamptr->R1>>2,lamptr->G1>>2,lamptr->B1>>2 );
		}
	break;

	case BLASTER:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->T01 = random(6)+2;
			lamptr->ltime1 = 0;
			}
		iprintf("BLASTER\r\n");
		newFA = FALSE;
		}
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 +=1;
		if (lamptr->ltime1 >lamptr->T01 )
			{
			lamptr->ltime1 = 0;
			lamptr->R1 = random(1024);
			lamptr->G1 = random(1024);
			lamptr->B1 = random(1024);
			setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
			}
		}
	break;

	case EXTREMEBLINK:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = random(803)-200;
			lamptr->G1 = random(803)-200;
			lamptr->B1 = random(803)-200;
			lamptr->R2 = random(803)-200;
			lamptr->G2 = random(803)-200;
			lamptr->B2 = random(803)-200;
			lamptr->R3 = random(803)-200;
			lamptr->G3 = random(803)-200;
			lamptr->B3 = random(803)-200;
			lamptr->ltime1=0;
			lamptr->T01 = 32;
			}
		iprintf("EXTREMEBLINK\r\n");
		newFA = FALSE;
		}
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempINT161 = 0;
		tempINT162 = 0;
		tempINT163 = 0;
		if (lamptr->ltime1 == 0)
			{
			tempINT161 = lamptr->R1;
			tempINT162 = lamptr->G1;
			tempINT163 = lamptr->B1;
			}
		if (lamptr->ltime1 == 8)
			{
			tempINT161 = lamptr->R2;
			tempINT162 = lamptr->G2;
			tempINT163 = lamptr->B2;
			}
		if (lamptr->ltime1 == 16)
			{
			tempINT161 = lamptr->R3;
			tempINT162 = lamptr->G3;
			tempINT163 = lamptr->B3;
			}
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 > lamptr->T01)
			lamptr->ltime1 = 0;
		setlight(i+start,tempINT161,tempINT162,tempINT163);
		}
	break;

	case FASTGREENSTROBE:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = random(8)+3;
			lamptr->T01 = tempINT161;
			tempINT161 += random(8)+3;
			lamptr->T12 = tempINT161;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("FASTGREENSTROBE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			{
			if (lamptr->G1 == 0)
				{
				lamptr->G1 = 1023;
				setlight(i+start,0,1023,0);
				}
			else
				{
				lamptr->G1 = 0;
				setlight(i+start,0,0,0);
				}
			}
		if (lamptr->ltime1 < lamptr->T12)
			lamptr->ltime1 += 1;
		else
			{
			lamptr->ltime1 = 0;
			tempINT161 = random(8)+3;
			}
		}
	break;

	case BLASTER2:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->T01 = random(10)+10;
			tempINT162 = random(10)+10;
			lamptr->ltime2 = tempINT162;
			lamptr->T12 = lamptr->T01 + tempINT162 + random(10);
			lamptr->ltime1 = 0;
			tempINT161 = random(1223) - 200;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->R1 = tempINT161<<4;
			tempINT161 = random(1223) - 200;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->G1 = tempINT161<<4;
 			tempINT161 = random(1223) - 200;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->B1 = tempINT161<<4;
			lamptr->R2 = lamptr->R1/lamptr->ltime2;
			lamptr->G2 = lamptr->G1/lamptr->ltime2;
			lamptr->B2 = lamptr->B1/lamptr->ltime2;
			}
		iprintf("BLASTER2\r\n");
		newFA = FALSE;
		}
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 >lamptr->T01 )
			{
			if (lamptr->R1 > lamptr->R2)
				lamptr->R1 -= lamptr->R2;
			if (lamptr->G1 > lamptr->G2)
				lamptr->G1 -= lamptr->G2;
			if (lamptr->B1 > lamptr->B2)
				lamptr->B1 -= lamptr->B2;
			}

		lamptr->ltime1 +=1;
		if (lamptr->ltime1 >= lamptr->T12)
			{
			lamptr->T01 = random(10)+10;
			tempINT162 = random(10)+10;
			lamptr->ltime2 = tempINT162;
			lamptr->T12 = lamptr->T01 + tempINT162 + random(10);
			lamptr->ltime1 = 0;
			tempINT161 = random(1223) - 200;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->R1 = tempINT161<<4;
			tempINT161 = random(1223) - 200;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->G1 = tempINT161<<4;
 			tempINT161 = random(1223) - 200;
			if (tempINT161 <0)
				tempINT161 = 0;
			lamptr->B1 = tempINT161<<4;
			lamptr->R2 = lamptr->R1/lamptr->ltime2;
			lamptr->G2 = lamptr->G1/lamptr->ltime2;
			lamptr->B2 = lamptr->B1/lamptr->ltime2;
			}
		setlight(i+start, lamptr->R1>>4 , lamptr->G1>>4, lamptr->B1>>4);
		}
	break;

	case SPOOKYFLICKER:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->B1 = 0;
			lamptr->G1 = 512;
			lamptr->G2 = 5;
			}
		iprintf("SPOOKYFLICKER\r\n");
		newFA = FALSE;
		}
	for(i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->G1>768)
			lamptr->G2 = -random(6)-2;
		if (lamptr->G1 < 256)
			lamptr->G2 = random(6)+2;
		lamptr->G1 += lamptr->G2;
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case SPOOKYWHITEFLICKER:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->B1 = 0;
			lamptr->G1 = 512;
			lamptr->G2 = 5;
			}
		iprintf("SPOOKYWHITEFLICKER\r\n");
		newFA = FALSE;
		}
	for(i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->G1>768)
			lamptr->G2 = -random(6)-2;
		if (lamptr->G1 < 256)
			lamptr->G2 = random(6)+2;
		lamptr->G1 += lamptr->G2;
		setlight(i+start,lamptr->G1, lamptr->G1, lamptr->G1);
		}
	break;

	case STROBEA:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			lamptr->T01 = 1;
			lamptr->T12 = random(20)+4;
			// clear ltime1;
			lamptr->ltime1 = 0;
			}
		iprintf("STROBEA\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->ltime1 < lamptr->T01)
			setlight(i+start,1023,1023,1023);
		else
			setlight(i+start,0,0,0);
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 >= lamptr->T12)
			lamptr->ltime1 = 0;
		}
	break;

	case INSANE:
		if(newFA == TRUE)
		{
			iprintf("insane\r\n");
			newFA = FALSE;
		}
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			tempINT161 = random(6);
			if (tempINT161 == 0 )
				{
				lamptr->R2 = random(1023);
				}
			if (tempINT161 == 1)
				{
				lamptr->G2 = random(1023);
				}
			if (tempINT161 == 2)
				{
				lamptr->B2 = random(1023);
				}
			setlight(i+start, lamptr->R2, lamptr->G2, lamptr->B2 );
			}
	break;

	case SOLIDGREEN:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->B1 = 0;
			lamptr->G1 = 512;
			}
		iprintf("SOLIDGREEN\r\n");
		newFA = FALSE;
		}
	for(i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case OOZE:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			lamptr->R1 = 512;
			lamptr->R2 = random(400)+600;
			lamptr->R3 = random(10)+2;
			}
		iprintf("OOZE\r\n");
		newFA = FALSE;
		}
	for(i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->R1> lamptr->R2)
			{
			if (lamptr->R3>0)
				{
				lamptr->R2 = random(100)+100;
				lamptr->R3 = -random(10)-2;
				}
			}
		if (lamptr->R1< lamptr->R2)
			{
			if (lamptr->R3<0)
				{
				lamptr->R2 = random(500)+500;
				lamptr->R3 = random(10)+2;
				}
			}
		lamptr->R1 += lamptr->R3;
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case REDFLAME:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			lamptr->R1 = 512;
			lamptr->R2 = 900;
			lamptr->R3 = 25;
			}
		iprintf("REDFLAME\r\n");
		newFA = FALSE;
		}
	for(i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->R1> lamptr->R2)
			{
			if (lamptr->R3 >0)
				lamptr->R2 = 100; // target brightness
			lamptr->R3 = -random(20)-20;
			}
		if (lamptr->R1< lamptr->R2)
			{
			if (lamptr->R3 <0)
				lamptr->R2 = 900;
			lamptr->R3 = random(20)+20;
			}
		lamptr->R1 += lamptr->R3;
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case REDGREENSPOOKY:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->B1 = 0;
			tempINT161=random(100);
			if (tempINT161<50)
				{
				lamptr->G1 = 512;
				lamptr->G2 = 5;
				lamptr->R1 = 0;
				lamptr->R2 = 0;
				}
			else
				{
				lamptr->R1 = 512;
				lamptr->R2 = 5;
				lamptr->G1 = 0;
				lamptr->G2 = 0;
				}
			}
		iprintf("REDGREENSPOOKY\r\n");
		newFA = FALSE;
		}
	for(i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->G1>768)
			lamptr->G2 = -random(6)-2;
		if ((lamptr->G1 < 256) && (lamptr->G2 != 0))
			lamptr->G2 = random(6)+2;
		lamptr->G1 += lamptr->G2;
		if (lamptr->R1>768)
			lamptr->R2 = -random(6)-2;
		if ((lamptr->R1 < 256) && (lamptr->R2 != 0))
			lamptr->R2 = random(6)+2;
		lamptr->R1 += lamptr->R2;
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case BLUESPOOKY:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 512;
			lamptr->B2 = 5;
			}
		iprintf("BLUESPOOKY\r\n");
		newFA = FALSE;
		}
	for(i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->B1>768)
			lamptr->B2 = -random(6)-2;
		if (lamptr->B1 < 256)
			lamptr->B2 = random(6)+2;
		lamptr->B1 += lamptr->B2;
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case FLOWY:
	if (newFA == TRUE)
		{
		lamp[start].R3 = random(1000)<<4;
		lamp[start].G3 = random(1000)<<4;
		lamp[start].B3 = random(1000)<<4;
		if (random(100)<20)
			lamp[start].R3 = 0;
		if (random(100)<20)
			lamp[start].G3 = 0;
		if (random(100)<20)
			lamp[start].B3 = 0;
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->B1 = 0;
			lamptr->G1 = 0;
			lamptr->T01 = (5*i) + 20;
// Put the iteration adds in temp for later
			lamp[i+start].R2 = (lamp[start].R3 - 0)/20;
			lamp[i+start].G2 = (lamp[start].G3 - 0)/20;
			lamp[i+start].B2 = (lamp[start].B3 - 0)/20;
			}
		iprintf("FLOWY\r\n");
		newFA = FALSE;
		}
	for(i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		lamptr->T01 -= 1; // COUNTING DOWN
		if (lamptr->T01 < 20)
			{
			lamptr->R1 += lamp[i+start].R2;
			lamptr->B1 += lamp[i+start].G2;
			lamptr->G1 += lamp[i+start].B2;
			if (lamptr->T01 <=0)
				{
				lamptr->R2 = (lamp[start].R3 - lamptr->R1)/20;
				lamptr->G2 = (lamp[start].G3 - lamptr->G1)/20;
				lamptr->B2 = (lamp[start].B3 - lamptr->B1)/20;
				lamptr->T01 = num * 5; // hold steady 20 seconds after turning on
				if (i==(num-1)) // all lights have changed
					{// generate new globals to flow to
					lamp[start].R3 = random(1000)<<4;
					lamp[start].G3 = random(1000)<<4;
					lamp[start].B3 = random(1000)<<4;
					if (random(100)<20)
						lamp[start].R3 = 0;
					if (random(100)<20)
						lamp[start].G3 = 0;
					if (random(100)<20)
						lamp[start].B3 = 0;

					}
				}
			}
		setlight(i+start,lamptr->R1>>4, lamptr->G1>>4, lamptr->B1>>4);
		}
	break;

	case GREENPULSE:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->B1 = 0;
			lamptr->G1 = 800;
			}
		iprintf("GREENPULSE\r\n");
		newFA = FALSE;
		lamp[start].ltime1 = 0;
		}
	clearlights(start,num);
	if (lamp[start].ltime1++ >10)
		{
		tempINT161 = random(num)+start; // start added
		lamp[tempINT161].G1 = 800;
		lamp[start].ltime1 = 0;
		if (num>10)
			{
			for (j=0; j<num/10;j++)
				{
				tempINT161 = random(num)+start; // start added
				lamp[tempINT161].G1 = 800;
				lamp[start].ltime1 = 0;
				}
			}
		}
	for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			if (lamptr->G1 >= 8 )
				lamptr->G1 -= 8;
			setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
			}
	break;

	case MUTATE3:
	if (newFA == TRUE)
		{
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			tempINT161 = random(350) - 100;
			if (tempINT161 < 0)
				tempINT161 = 0;
			lamptr->R3 = tempINT161<<2;
			tempINT161 = random(350) - 100;
			if (tempINT161 < 0)
				tempINT161 = 0;
			lamptr->G3 = tempINT161<<2;
 			tempINT161 = random(350) - 100;
			if (tempINT161 < 0)
				tempINT161 = 0;
			lamptr->B3 = tempINT161<<2;
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			}
		iprintf("MUTATE3\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		if (abs(lamptr->R1 - lamptr->R3) <4)
			{
			//lamptr->R3 = random(250)<<2;
			tempINT161 = random(350) - 100;
			if (tempINT161 < 0)
				tempINT161 = 0;
			lamptr->R3 = tempINT161<<2;
			}
		if (lamptr->R1 < lamptr->R3)
			lamptr->R1 += 4;
		else
			lamptr->R1 -= 4;
		if (abs(lamptr->G1 - lamptr->G3) <4)
			{
			//lamptr->G3 = random(250)<<2;
			tempINT161 = random(350) - 100;
			if (tempINT161 < 0)
				tempINT161 = 0;
			lamptr->G3 = tempINT161<<2;
			}
		if (lamptr->G1 < lamptr->G3)
			lamptr->G1 += 4;
		else
			lamptr->G1 -= 4;
		if (abs(lamptr->B1 - lamptr->B3) <4)
			{
			//lamptr->B3 = random(250)<<2;
			tempINT161 = random(350) - 100;
			if (tempINT161 < 0)
				tempINT161 = 0;
			lamptr->B3 = tempINT161<<2;
			}
		if (lamptr->B1 < lamptr->B3)
			lamptr->B1 += 4;
		else
			lamptr->B1 -= 4;

		setlight(i+start, lamptr->R1>>2,lamptr->G1>>2,lamptr->B1>>2 );
		}
	break;

	case ALTERNATOR:
		if (newFA == TRUE)
			{
			for (i=0; i<4; i++)
				{
				lamptr = &lamp[i+start];
				lamptr->R1 = 0;
				lamptr->G1 = 0;
				lamptr->B1 = 0;
				if (i == 1)
				    {
					lamptr->R1 = random(1200)-300;
					if (lamptr->R1 <0)
					    lamptr->R1 = 0;
					lamptr->G1 = random(1200)-300;
					if (lamptr->G1 <0)
					    lamptr->G1 = 0;
					lamptr->B1 = random(1200)-300;
					if (lamptr->B1 <0)
					    lamptr->B1 = 0;

				    }
				if (i == 3)
				    {
					lamptr->R1 = 800;
					lamptr->G1 = 800;
					lamptr->B1 = 800;
				    }
				}
			j=0;
			for (i=4; i<num;i++)
			    {
			    lamptr = &lamp[i+start];
			    lamptr2 = &lamp[j+start];
			    lamptr->R1 = lamptr2->R1;
			    lamptr->G1 = lamptr2->G1;
			    lamptr->B1 = lamptr2->B1;
			    j+= 1;
			    if (j>= 4)
				j=0;
			    }
			iprintf("ALTERNATOR\r\n");
			newFA = FALSE;
			}
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			setlight(i+start, lamptr->R1,lamptr->G1,lamptr->B1 );
			}
		break;

	case BLUEPULSE:
	if (newFA == TRUE)
		{
		for(i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 0;
			lamptr->B1 = 800;
			lamptr->G1 = 0;
			}
		iprintf("BLUEPULSE\r\n");
		newFA = FALSE;
		lamp[start].ltime1 = 0;
		}
	clearlights(start,num);
	if (lamp[start].ltime1++ >10)
		{
		tempINT161 = random(num)+start; // start added
		lamp[tempINT161].B1 = 800;
		lamp[start].ltime1 = 0;
		if (num>10)
			{
			for (j=0; j<num/10;j++)
				{
				tempINT161 = random(num)+start; // start added
				lamp[tempINT161].B1 = 800;
				lamp[start].ltime1 = 0;
				}
			}
		}
	for(i=0; i<num; i++)
			{
		lamptr = &lamp[i+start];
			if (lamptr->B1 >= 8 )
				lamptr->B1 -= 8;
			setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
			}
	break;

	case REDGREENCHASE:
	if (newFA == TRUE)
		{
		j = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = 0;
			lamptr->T01 = tempINT161;
			tempINT161 += 40; // two second fade
			lamptr->T12 = tempINT161;
			tempINT161 += 40;
			lamptr->T23 = tempINT161;

			lamptr->ltime1 = j * 20; // each light is at a different phase

			j+= 1;
			if (j>=8)
				j=0;

			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			lamptr->R2 = 0;
			lamptr->G2 = 0;
			if (j<4)
			{
				lamptr->ltime1 = j * 20; // each light is at a different phase
				lamptr->R1 = 16368;
				lamptr->R2 = 16368/40;
			}
			else
			{
				lamptr->ltime1 = (j-4) * 20; // each light is at a different phase
				lamptr->G1 = 16368;
				lamptr->G2 = 16368/40;
			}
			lamptr->B2 = 0;
			}
		iprintf("REDGREENCHASE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T12)
			// followed by a fade to the color
			{
			lamptr->R1 -= lamptr->R2;
			tempRed = lamptr->R1>>4;
			lamptr->G1 -= lamptr->G2;
			tempGrn = lamptr->G1>>4;
			lamptr->B1 -= lamptr->B2;
			tempBlu = lamptr->B1>>4;
			}
		else
		{
			tempRed = 0;
			tempGrn = 0;
			tempBlu = 0;
		}

		if (lamptr->ltime1 > lamptr->T23)
		{
			if (lamptr->R2 !=0) // change to green from red
			{
				lamptr->R1 = 0;
				lamptr->R2 = 0;
				lamptr->G1 = 16368;
				lamptr->G2 = 16368/40;
			}
			else
			{
				lamptr->G1 = 0;
				lamptr->G2 = 0;
				lamptr->R1 = 16368;
				lamptr->R2 = 16368/40;
			}
			lamptr->B1 = 0;
			lamptr->ltime1 = 0;
		}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}
	break;

	case ALTREDWHITE:
			if (newFA == TRUE)
				{
				for (i=0; i<4; i++)
					{
					lamptr = &lamp[i+start];
					lamptr->R1 = 0;
					lamptr->G1 = 0;
					lamptr->B1 = 0;
					if (i == 1)
					    {
						lamptr->R1 = 1023;
					    }
					if (i == 3)
					    {
						lamptr->R1 = 800;
						lamptr->G1 = 800;
						lamptr->B1 = 800;
					    }
					}
				j=0;
				for (i=4; i<num;i++)
				    {
				    lamptr = &lamp[i+start];
				    lamptr2 = &lamp[j+start];
				    lamptr->R1 = lamptr2->R1;
				    lamptr->G1 = lamptr2->G1;
				    lamptr->B1 = lamptr2->B1;
				    j+= 1;
				    if (j>= 4)
					j=0;
				    }
				iprintf("ALTERNATOR\r\n");
				newFA = FALSE;
				}
			for (i=0; i<num; i++)
				{
				lamptr = &lamp[i+start];
				setlight(i+start, lamptr->R1,lamptr->G1,lamptr->B1 );
				}
			break;

	case STEADYWARM:
	if (newFA == TRUE)
		{
		j = 0;
		for (i=0;i<num;i++)
			{
			lamptr = &lamp[i+start];
			lamptr->R1 = 128; // white and some orange, dimmed
			lamptr->G1 = 110;
			lamptr->B1 = 61;
			}
		iprintf("STEADYWARM\r\n");
		newFA = FALSE;
		}
	lamptr = &lamp[start];
	for (i=0;i<num;i++)
		{
		//lamptr = &lamp[i+start];
		lamptr++;
		setlight(i+start,lamptr->R1,lamptr->G1,lamptr->B1);
		}

	break;

	case PINGPONG:
//	reRoute = MUTATE;
	if (newFA == TRUE)
		{
		for (i=0; i<3; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime2 = random(32)+6; // speed
			lamptr->ltime1 = 8<<5; // position
			}
		iprintf("PINGPONG\r\n");
		newFA = FALSE;
		}
	for (i=0; i<3; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += lamptr->ltime2;
		tempINT161 = ((lamptr->ltime1)>>5); // index without start
		if(tempINT161 >= num-6)
			{
			lamptr->ltime2 = -lamptr->ltime2;
			lamptr->ltime1 += lamptr->ltime2;
			}
		if(tempINT161 <= 2)
			{
			lamptr->ltime2 = abs(lamptr->ltime2);
			lamptr->ltime1 += lamptr->ltime2;
			}
		lamp[tempINT161+start].R1 = 0;
		lamp[tempINT161+start].G1 = 0;
		lamp[tempINT161+start].B1 = 0;
		if (i==0)
			lamp[tempINT161+start].R1 = 252;
		if (i==1)
			lamp[tempINT161+start].G1 = 252;
		if (i==2)
			lamp[tempINT161+start].B1 = 252;
// paint four light
		for (j=0;j<4;j+=1)
			{
			setlight(tempINT161+start+j,lamptr->R1, lamptr->G1, lamptr->B1);
			}
		}
	for (i=0;i<num;i++)
		{
		lamptr = &lamp[i+start];
		if (lamptr->R1 >= 12) lamptr->R1 -= 12;
		else lamptr->R1 = 0;
		if (lamptr->G1 >= 12) lamptr->G1 -= 12;
		else lamptr->G1 = 0;
		if (lamptr->B1 >= 12) lamptr->B1 -= 12;
		else lamptr->B1 = 0;
		setlight(i+start,lamptr->R1, lamptr->G1, lamptr->B1);
		}

	break;


	case YELLOWBLENDCHASE:
	if (newFA == TRUE)
		{
		j = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			// set T01 to time on and T02 to time off
			tempINT161 = 0;
			lamptr->T01 = tempINT161;
			tempINT161 += 60; // two second fade red to green
			lamptr->T12 = tempINT161;
			tempINT161 += 60; // two second fade green to red
			lamptr->T23 = tempINT161;

			lamptr->ltime1 = j * 20; // each light is at a different phase

			j+= 1;
			if (j>=6)
				j=0;

			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			lamptr->R2 = 0;
			lamptr->G2 = 0;
			if (j<3)
			{
				lamptr->R1 = 16368;
				lamptr->G1 = 0;
				lamptr->G2 = -16368/60;
				lamptr->R2 = 16368/60;
			}
			else
			{
				lamptr->R1 = 0;
				lamptr->G1 = 16368;
				lamptr->G2 = 16368/60;
				lamptr->R2 = -16368/60;
			}
			lamptr->B2 = 0;
			}
		iprintf("YELLOWBLENDCHASE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 == lamptr->T12)
		{
			lamptr->R1 = 0;
			lamptr->G1 = 16368;
			lamptr->G2 = 16368/60;
			lamptr->R2 = -16368/60;
		}
		if (lamptr->ltime1 <= lamptr->T23)
			// followed by a fade to the color
			{
			lamptr->R1 -= lamptr->R2;
			tempRed = lamptr->R1>>4;
			lamptr->G1 -= lamptr->G2;
			tempGrn = lamptr->G1>>4;
			lamptr->B1 -= lamptr->B2;
			tempBlu = lamptr->B1>>4;
			}
		if (lamptr->ltime1 == lamptr->T23) // restart
		{
			lamptr->R1 = 16368;
			lamptr->G1 = 0;
			lamptr->G2 = -16368/60;
			lamptr->R2 = 16368/60;
			lamptr->ltime1 = 0;
		}
		setlight(i+start,tempRed,tempGrn,tempBlu);
		}
	break;

	case NADA:
	    // do absolutely nothing
	    newFA = FALSE;
	    break;
//

// sIX STICKS OF 25 LIGHTS EACH - 150 pixels across six sticks

	case STICKSPIN:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		j=0;
		tempINT161 = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = j;
			lamptr->T01 = 50; // turn on
			lamptr->T12 = 60; // turn off

			tempINT161 += 1;
			if (tempINT161 >= 25) // stick filled
				{
				tempINT161 = 0;
				j+= 10;
				}
			}
		iprintf("STICKSPIN\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempINT161 = 0;
		tempINT162 = 0;
		tempINT163 = 0;
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 > lamptr->T01)
			{
				tempINT161 = lamp[start].R2;
				tempINT162 = lamp[start].G2;
				tempINT163 = lamp[start].B2;
				if (lamptr->ltime1 >= lamptr->T12)
					lamptr->ltime1 = 0; // reset
			}
		setlight(i+start, tempINT161,tempINT162,tempINT163 );
		}
	break;

// BREAK
	case STICKUP:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		j=0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = j;
			j+=10;
			if (j>20)
				j=0;
			lamptr->T01 = 80; // turn on
			lamptr->T12 = 90; // turn off
			}
		iprintf("STICKUP\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		tempINT161 = 0;
		tempINT162 = 0;
		tempINT163 = 0;
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 > lamptr->T01)
			{
				tempINT161 = lamp[start].R2;
				tempINT162 = lamp[start].G2;
				tempINT163 = lamp[start].B2;
				if (lamptr->ltime1 >= lamptr->T12)
					lamptr->ltime1 = 0; // reset at 4.5sec
			}
		setlight(i+start, tempINT161,tempINT162,tempINT163 );
		}
	break;

	case STICKUPFADE:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		lamp[start].R3 = lamp[start].R2 / 20;
		lamp[start].G3 = lamp[start].G2 / 20;
		lamp[start].B3 = lamp[start].B2 / 20;
		j=0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = j;
			j+=1;
			if (j>=PIXELSSTICK)
				j=0;
			lamptr->T01 = 10; // turn off
			lamptr->T12 = 30; // turn off fade
			lamptr->T23 = 100;
			}
		iprintf("STICKUPFADE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T12)
			{
			lamptr->R1 -= lamp[start].R3;
			lamptr->G1 -= lamp[start].G3;
			lamptr->B1 -= lamp[start].B3;
			}
		if (lamptr->ltime1 < lamptr->T01)
			{
			lamptr->R1 = lamp[start].R2;
			lamptr->G1 = lamp[start].G2;
			lamptr->B1 = lamp[start].B2;
			}
		if (lamptr->ltime1 >= lamptr->T23)
			lamptr->ltime1 = 0; // reset at 4.5sec
		setlight(i+start, lamptr->R1, lamptr->G1, lamptr->B1);
		}
	break;

	case STICKDOWNFADE:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		lamp[start].R3 = lamp[start].R2 / 16;
		lamp[start].G3 = lamp[start].G2 / 16;
		lamp[start].B3 = lamp[start].B2 / 16;
		j=PIXELSSTICK-1;
		tempINT161 = random(30);
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->T01 = 10; // turn off
			lamptr->T12 = 26; // turn off fade
			lamptr->T23 = 80;
			lamptr->ltime1 = j+tempINT161;
			j-= 1;
			if (j<0)
				{
				tempINT161 = random(30);
				j=PIXELSSTICK-1;
				}
			}
		iprintf("STICKDOWNFADE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T01)
			{
			lamptr->R1 = lamp[start].R2;
			lamptr->G1 = lamp[start].G2;
			lamptr->B1 = lamp[start].B2;
			}
		else if (lamptr->ltime1 < lamptr->T12)
			{
			lamptr->R1 -= lamp[start].R3;
			lamptr->G1 -= lamp[start].G3;
			lamptr->B1 -= lamp[start].B3;
			}
		if (lamptr->ltime1 == lamptr->T12)
			{
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			}
		if (lamptr->ltime1 >= lamptr->T23)
			lamptr->ltime1 = 0; // reset at 4.5sec
		setlight(i+start, lamptr->R1,lamptr->G1,lamptr->B1 );
		}
	break;

	case STICKTHRUMDOWN:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		lamp[start].R3 = lamp[start].R2 / 32;
		lamp[start].G3 = lamp[start].G2 / 32;
		lamp[start].B3 = lamp[start].B2 / 32;
		j=PIXELSSTICK-1;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = j;
			lamptr->T01 = 10; // turn off
			lamptr->T12 = 42; // turn off fade
			lamptr->T23 = 100;
			j-=1;
			if (j<0)
				j=PIXELSSTICK-1;
			}
		iprintf("STICKTHRUMDOWN\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T12)
			{
			lamptr->R1 -= lamp[start].R3;
			lamptr->G1 -= lamp[start].G3;
			lamptr->B1 -= lamp[start].B3;
			}
		if (lamptr->ltime1 < lamptr->T01)
			{
			lamptr->R1 = lamp[start].R2;
			lamptr->G1 = lamp[start].G2;
			lamptr->B1 = lamp[start].B2;
			}
		if (lamptr->ltime1 == lamptr->T12)
			{
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			}
		if (lamptr->ltime1 >= lamptr->T23)
			lamptr->ltime1 = 0; // reset at 4.5sec
		setlight(i+start, lamptr->R1, lamptr->G1, lamptr->B1 );
		}
	break;

	case STICKTHRUMDOWNRAND:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		lamp[start].R3 = lamp[start].R2 / 32;
		lamp[start].G3 = lamp[start].G2 / 32;
		lamp[start].B3 = lamp[start].B2 / 32;
		j=PIXELSSTICK-1;
		tempINT161 = random(120);
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = tempINT161  +  j;
			lamptr->T01 = 10; // turn off
			lamptr->T12 = 42; // turn off fade
			lamptr->T23 = 140;
			j-=1;
			if (j<0)
				{
				tempINT161 = random(120);
				j=PIXELSSTICK-1;
				}
			}
		iprintf("STICKTHRUMDOWNRAND\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T12)
			{
			lamptr->R1 -= lamp[start].R3;
			lamptr->G1 -= lamp[start].G3;
			lamptr->B1 -= lamp[start].B3;
			}
		if (lamptr->ltime1 < lamptr->T01)
			{
			lamptr->R1 = lamp[start].R2;
			lamptr->G1 = lamp[start].G2;
			lamptr->B1 = lamp[start].B2;
			}
		if (lamptr->ltime1 == lamptr->T12)
			{
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			}
		if (lamptr->ltime1 >= lamptr->T23)
			lamptr->ltime1 = 0; // reset at 4.5sec
		setlight(i+start, lamptr->R1, lamptr->G1, lamptr->B1 );
		}
	break;

	case STICKSPINFADE:
	if (newFA == TRUE)
		{
		lamp[start].R2 = random(1224); // set color for all lights
		if (lamp[start].R2 >1023)
			lamp[start].R2 = 0;
		lamp[start].G2 = random(1224);
		if (lamp[start].G2 >1023)
			lamp[start].G2 = 0;
		lamp[start].B2 = random(1224);
		if (lamp[start].B2 >1023)
			lamp[start].B2 = 0;
		lamp[start].R3 = lamp[start].R2 / 16;
		lamp[start].G3 = lamp[start].G2 / 16;
		lamp[start].B3 = lamp[start].B2 / 16;
		j=0;
		tempINT161 = 0;
		for (i=0; i<num; i++)
			{
			lamptr = &lamp[i+start];
			lamptr->ltime1 = j;
			lamptr->T01 = 4; // on till
			lamptr->T12 = 20; // fade till
			lamptr->T23 = 32; // start over
			tempINT161 += 1;
			if (tempINT161 >= PIXELSSTICK)
				{
				tempINT161 = 0;
				j+= PIXELSSTICK;
				}
			}
		iprintf("STICKSPINFADE\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	for (i=0; i<num; i++)
		{
		lamptr = &lamp[i+start];
		lamptr->ltime1 += 1;
		if (lamptr->ltime1 < lamptr->T12)
			{
			lamptr->R1 -= lamp[start].R3;
			lamptr->G1 -= lamp[start].G3;
			lamptr->B1 -= lamp[start].B3;
			}
		if (lamptr->ltime1 < lamptr->T01)
			{
			lamptr->R1 = lamp[start].R2;
			lamptr->G1 = lamp[start].G2;
			lamptr->B1 = lamp[start].B2;
			}
		if (lamptr->ltime1 == lamptr->T12)
			{
			lamptr->R1 = 0;
			lamptr->G1 = 0;
			lamptr->B1 = 0;
			}
		if (lamptr->ltime1 >= lamptr->T23)
			lamptr->ltime1 = 0;
		setlight(i+start, lamptr->R1, lamptr->G1, lamptr->B1 );
		}
	break;

	case ARCHTEST: // a single arch controlled by lamp 0; VER
	if (newFA == TRUE)
		{
		tempINT161 = random(1223)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].R1 = tempINT161;
		tempINT161 = random(1223)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].G1 = tempINT161;
		tempINT161 = random(1223)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].B1 = tempINT161;
		arch[0].ltime1 = 1; // time accumulator
		arch[0].ltime2 = 1; // time incrementer
		pixcount = 1;
		iprintf("ARCHTEST\r\n");
		newFA = FALSE;
		}
	clearlights(start,num);
	arch[0].ltime1 += arch[0].ltime2;
	if (arch[0].ltime1 >= PIXELSARCH-1)
			{
			arch[0].ltime2 = -1;
			}
	if (arch[0].ltime1<=1)
			{
			arch[0].ltime2 = 1;
			}
	setarch(start, 0, 0, arch[0].R1, arch[0].G1, arch[0].B1, arch[0].ltime1);
	setarch(start, 1, 0, arch[0].R1, arch[0].G1, arch[0].B1, arch[0].ltime1);
	setarch(start, 2, 0, arch[0].R1, arch[0].G1, arch[0].B1, arch[0].ltime1);

	break;

	case ARCHSINGLEFADE: // a single light with background color
	if (newFA == TRUE)
		{
		// ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		tempINT161 = random(600)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].R1 = tempINT161;
		tempINT161 = random(600)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].G1 = tempINT161;
		tempINT161 = random(600)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].B1 = tempINT161;
		arch[0].ltime1 = 0; // time accumulator
		arch[0].T01 = 3; // time incrementer

		tempINT161 = random(1223)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[1].R1 = tempINT161;
		tempINT161 = random(1223)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[1].G1 = tempINT161;
		tempINT161 = random(1223)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[1].B1 = tempINT161;
		arch[1].ltime1 = 0; // time accumulator
		arch[1].begin = 0; // which light to turn on
		for (i=start; i<(start+num); i++)
		    { // SET ALL BACKGROUND COLOR
		    lamp[i].R1 = arch[0].R1;
		    lamp[i].G1 = arch[0].G1;
		    lamp[i].B1 = arch[0].B1;
		    // set the amount to change
		    lamp[i].R2 = (arch[0].R1 - arch[1].R2) / 8;
		    lamp[i].G2 = (arch[0].G1 - arch[1].G2) / 8;
		    lamp[i].B2 = (arch[0].B1 - arch[1].B2) / 8;
		    lamp[i].ltime1 = 0;
		    }
		pixcount = random(5)+2;
		arch[1].T01 = random(pixcount)+1;
		iprintf("ARCHSINGLEFADE\r\n");
		newFA = FALSE;
		} // newFA
	for (i=start; i<(start+num); i++)
		{ // move lights towards background color
		if (lamp[i].ltime1>0)
			{
			lamp[i].R1 -= lamp[i].R2;
			lamp[i].G1 -= lamp[i].G2;
			lamp[i].B1 -= lamp[i].B2;
			lamp[i].ltime1 -= 1;
			}
		else
			{ // set lights to background color
			lamp[i].R1 = arch[0].R1;
			lamp[i].G1 = arch[0].G1;
			lamp[i].B1 = arch[0].B1;
			}
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
	arch[1].begin+=arch[1].T01;
	if (arch[1].begin >= PIXELSARCH)
		{
		arch[1].begin = -pixcount;
		arch[1].archnum = random(NUMARCHS*2);
		}
	setarchfade(start, arch[1].archnum, arch[1].begin, arch[1].R1, arch[1].G1, arch[1].B1, 8, pixcount);
	break;

	case ARCHMULTIBACK: // 4 lights with background color VER
	if (newFA == TRUE)
		{
		 // ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		tempINT161 = random(600)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].R1 = tempINT161;
		tempINT161 = random(600)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].G1 = tempINT161;
		tempINT161 = random(600)-200;
		if (tempINT161 <0)
				tempINT161 = 0;
		arch[0].B1 = tempINT161;
		arch[0].ltime1 = 0; // time accumulator
		arch[0].T01 = 3; // time incrementer
		for(i=1; i<5; i++)
			{
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
					tempINT161 = 0;
			arch[i].R1 = tempINT161;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
					tempINT161 = 0;
			arch[i].G1 = tempINT161;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
					tempINT161 = 0;
			arch[i].B1 = tempINT161;

			arch[i].begin = random(PIXELSARCH); // which light to turn on
			arch[i].archnum = random(NUMARCHS); // which arch
			}
		pixcount = random(5)+2;
		iprintf("ARCHMULTIBACK\r\n");
		newFA = FALSE;
		} // newFA
    for (i = start; i<start+num; i++)
		{
 // set lights to background color
		lamp[i].R1 = arch[0].R1;
		lamp[i].G1 = arch[0].G1;
		lamp[i].B1 = arch[0].B1;
		}
// Draw the four lines
    for(i=1; i<5; i++)
		{
		arch[i].begin += pixcount;
		if (arch[i].begin >= PIXELSARCH)
			{
			arch[i].begin = -pixcount;
			arch[i].archnum = random(NUMARCHS*2);
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, pixcount);
		}
    for (i = start; i<start+num; i++)
		{
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    break;

	case ARCHFIREFLY: // 2 WHITE lights with background color VER
	if (newFA == TRUE)
		{
		i=0; // ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		arch[i].R1 = 0;
		arch[i].G1 = 0;
		arch[i].B1 = 0;
		arch[i].ltime1 = 0; // time accumulator
		for(i=1; i<3; i++)
			{
			arch[i].R1 = 1023;
			arch[i].G1 = 1023;
			arch[i].B1 = 1023;
			arch[i].ltime1 = 0; // time accumulator
			arch[i].begin = -2; // which lights to turn on
			arch[i].end = 0;
			arch[i].archnum = random(NUMARCHS*2); // which arch
			arch[i].ltime1 = 0; // time accumulator
			}
		pixcount = random(6)+3;
		iprintf("ARCHFIREFLY\r\n");
		newFA = FALSE;
		}
    	for (i = start; i<start+num; i++)
		{
 // set lights to background color
		lamp[i].R1 = arch[0].R1;
		lamp[i].G1 = arch[0].G1;
		lamp[i].B1 = arch[0].B1;
		}
    	for(i=1; i<3; i++)
		{
		arch[i].begin += pixcount;
		if (arch[i].begin >= PIXELSARCH)
			{
			arch[i].begin = -pixcount;
			arch[i].archnum = random(NUMARCHS*2);
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, pixcount);
		}// for
    for (i = start; i<start+num; i++)
		{
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    break;

	case ARCHREDGRN1: // 3 lights making 180 degree arcs VER
	if (newFA == TRUE)
		{
		 // ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		arch[0].R1 = 0;
		arch[0].G1 = 0;
		arch[0].B1 = 0;
		arch[0].ltime1 = 0; // time accumulator
		for(i=1; i<=NUMARCHS; i++)// for all 3 archs
			{
			arch[i].begin = -PIXELSARCH;
			if ((i==1)||(i==3)) //  make red altnernating arches
			{
				arch[i].R1 = 1023;
				arch[i].G1 = 0;
				arch[i].B1 = 0;
				arch[i].T01 = 0; // signals light is red
			}
			else
			{
				arch[i].G1 = 1023;
				arch[i].R1 = 0;
				arch[i].B1 = 0;
				arch[i].T01 = 1; // signals the light is green
			}
			arch[i].archnum = i-1; // which arch  start at 0
			arch[i].ltime1 = 0; // time accumulator
			}
		iprintf("ARCHREDGRN1\r\n");
		newFA = FALSE;
		}
	for(i=1; i<=NUMARCHS; i++) // all 3 arches
		{
		arch[i].ltime1+=1;
		if (arch[i].ltime1 >= 2)
			{
			arch[i].ltime1 = 0;
			arch[i].begin += 1;
			if (arch[i].begin >= 0)
				{
					arch[i].begin = -PIXELSARCH;
					if (arch[i].T01 == 0)
					{
						arch[i].R1 = 0;
						arch[i].G1 = 1023;
						arch[i].T01 = 1;
					}
					else
					{
						arch[i].R1 = 1023;
						arch[i].G1 = 0;
						arch[i].T01 = 0;
					}
				}
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, PIXELSARCH );
		}
	for (i = start; i<start+num; i++)
		{
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    break;

	case ARCHMULTIBACKVARY: // 4 lights with background color changing over time VER
	if (newFA == TRUE)
		{
		// ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		arch[0].T01 = 3; // time incrementer
		// set T01 to time between colors
		tempINT161 = random(100)+100; // 5-10 seconds
		arch[0].T12 = tempINT161;
		// set a color for the lamp -- shifted 5 places for math precision
		arch[0].R3 = (random(500) -150)*32;
		arch[0].R1 = 0;
		arch[0].G3 = (random(500) -150)*32;
		arch[0].G1 = 0;
		arch[0].B3 = (random(500) -150)*32;
		arch[0].B1 = 0;
		if (arch[0].R3 <0)
			arch[0].R3 = 0;
		if (arch[0].G3<0)
			arch[0].G3 = 0;
		if (arch[0].B3<0)
			arch[0].B3 = 0;
		// set the increments for the colors over the time period
		arch[0].R2 = arch[0].R3 / tempINT161;
		arch[0].G2 = arch[0].G3 / tempINT161;
		arch[0].B2 = arch[0].B3 / tempINT161;
		// clear ltime1;
		arch[0].ltime1 = 0;
		arch[0].ltime2 = 0;
		// set the 8 lights to their colors and positions.
		pixcount = random(5)+2;
		for(i=1; i<5; i++)
			{
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
					tempINT161 = 0;
			arch[i].R1 = tempINT161;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
					tempINT161 = 0;
			arch[i].G1 = tempINT161;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
					tempINT161 = 0;
			arch[i].B1 = tempINT161;
			arch[i].ltime1 = random(3); // time accumulator
			arch[i].begin = random(PIXELSARCH); // which light to turn on
			arch[i].archnum = random(NUMARCHS*2); // which arch
			}
		iprintf("ARCHMULTIBACKVARY\r\n");
		newFA = FALSE;
		} // end flagFA
// change background color
	if (arch[0].ltime2 < arch[0].T12)
		{
		arch[0].R1+=arch[0].R2;
		arch[0].G1+=arch[0].G2;
		arch[0].B1+=arch[0].B2;
		arch[0].ltime2 += 1;
		}
	else // change to new color
		{
		arch[0].R1 = arch[0].R3;
		arch[0].G1 = arch[0].G3;
		arch[0].B1 = arch[0].B3;
		arch[0].R3 = (random(500) -150)*32;
		arch[0].G3 = (random(500) -150)*32;
		arch[0].B3 = (random(500) -150)*32;
		if (arch[0].R3<0)
			arch[0].R3 = 0;
		if (arch[0].G3<0)
			arch[0].G3 = 0;
		if (arch[0].B3<0)
			arch[0].B3 = 0;
		tempINT161 = arch[0].T12;
		tempINT162 = arch[0].R3 - arch[0].R1;
		arch[0].R2 = tempINT162 / tempINT161;
		tempINT162 = arch[0].G3 - arch[0].G1;
		arch[0].G2 = tempINT162 / tempINT161;
		tempINT162 = arch[0].B3 - arch[0].B1;
		arch[0].B2 = tempINT162 / tempINT161;
		// clear ltime2;
		arch[0].ltime2 = 0;
		}
    for (i=start; i<start+num; i++)
		{
 // set lights to background color
		lamp[i].R1 = arch[0].R1>>5;
		lamp[i].G1 = arch[0].G1>>5;
		lamp[i].B1 = arch[0].B1>>5;
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    for(i=1; i<5; i++)
		{
		arch[i].ltime1+=1;
		arch[i].begin += 1;
		if (arch[i].begin >= PIXELSARCH)
			{
			arch[i].begin = -pixcount;
			arch[i].archnum = random(NUMARCHS*2);
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, pixcount);
		}
    break;

	case ARCHREDGRN2: // 3 lights making 180 degree arcs VER
	if (newFA == TRUE)
		{
	 // ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		arch[0].R1 = 0;
		arch[0].G1 = 1023;
		arch[0].B1 = 0;
		arch[0].ltime1 = 0; // time accumulator
		for(i=1; i<=(NUMARCHS); i++)// for all 3 archs
			{
			arch[i].R1 = 1023;
			arch[i].G1 = 0;
			arch[i].B1 = 0;
			arch[i].begin =  (16*i);
			arch[i].end = 16;
			arch[i].T01 = 1; // signals the light is green

			arch[i].archnum = i-1; // which arch
			arch[i].ltime1 = i-1; // time accumulator, distributed load
			}
		iprintf("ARCHREDGRN2\r\n");
		newFA = FALSE;
		}
    for (i = start; i<start+(NUMARCHS*PIXELSARCH); i++)
		{
 // set lights to background color
		lamp[i].R1 = arch[0].R1;
		lamp[i].G1 = arch[0].G1;
		lamp[i].B1 = arch[0].B1;
		}
    for(i=1; i<=(NUMARCHS); i++) // all 3 arches
		{
		arch[i].ltime1+=1;
		if (arch[i].ltime1 >= 3)
			{
			arch[i].ltime1 = 0;
			arch[i].begin += 1;
			if (arch[i].begin >=PIXELSARCH)
				{
				arch[i].begin = -16;
				arch[i].end = 16;
				}
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, arch[i].end);
		}
    for (i = start; i<start+num; i++)
		{
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    break;

	case ARCHOFF:
		if (newFA == TRUE)
		{
			iprintf("ARCHOFF\r\n");
			newFA = FALSE;
			clearlights(start,num);
		}
	break;

	case ARCHRAIN: // 3 blue lights with background color changing over time VER
	if (newFA == TRUE)
		{
		; // ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		arch[0].T01 = 4; // time incrementer
		// set T01 to time between colors
		tempINT161 = random(100)+100; // 5-10 seconds
		arch[0].T12 = tempINT161;
		// set a color for the lamp -- shifted 5 places for math precision
		arch[0].R3 = (random(500) -150)*32;
		arch[0].R1 = 0;
		arch[0].G3 = (random(500) -150)*32;
		arch[0].G1 = 0;
		arch[0].B3 = (random(500) -150)*32;
		arch[0].B1 = 0;
		if (arch[0].R3 <0)
			arch[0].R3 = 0;
		if (arch[0].G3<0)
			arch[0].G3 = 0;
		if (arch[0].B3<0)
			arch[0].B3 = 0;
		// set the increments for the colors over the time period
		arch[0].R2 = arch[i].R3 / tempINT161;
		arch[0].G2 = arch[i].G3 / tempINT161;
		arch[0].B2 = arch[i].B3 / tempINT161;
		// clear ltime1;
		arch[i].ltime1 = 0;
		arch[i].ltime2 = 0;
		// set the 4 lights to their colors and positions.
		for(i=1; i<4; i++)
			{
			arch[i].R1 = 0;
			arch[i].G1 = 0;
			arch[i].B1 = 768 * 32;
			arch[i].ltime1 = random(4); // time accumulator
			arch[i].begin = (PIXELSARCH / 2) - 3; // which light to turn on
			arch[i].archnum = random(NUMARCHS); // which arch
			}
		iprintf("ARCHRAIN\r\n");
		newFA = FALSE;
		} // end flagFA
// change background color
	if (arch[0].ltime2 < arch[0].T12)
		{
		arch[0].R1+=arch[0].R2;
		arch[0].G1+=arch[0].G2;
		arch[0].B1+=arch[0].B2;
		arch[0].ltime2 += 1;
		}
	else // change to new color
		{
		arch[0].R1 = arch[0].R3;
		arch[0].G1 = arch[0].G3;
		arch[0].B1 = arch[0].B3;
		arch[0].R3 = (random(500) -150)*32;
		arch[0].G3 = (random(500) -150)*32;
		arch[0].B3 = (random(500) -150)*32;
		if (arch[0].R3<0)
			arch[0].R3 = 0;
		if (arch[0].G3<0)
			arch[0].G3 = 0;
		if (arch[0].B3<0)
			arch[0].B3 = 0;
		tempINT161 = arch[0].T12;
		tempINT162 = arch[0].R3 - arch[0].R1;
		arch[0].R2 = tempINT162 / tempINT161;
		tempINT162 = arch[0].G3 - arch[0].G1;
		arch[0].G2 = tempINT162 / tempINT161;
		tempINT162 = arch[0].B3 - arch[0].B1;
		arch[0].B2 = tempINT162 / tempINT161;
		// clear ltime2;
		arch[0].ltime2 = 0;
		}
    for (i = start; i<start+num; i++)
		{
 // set lights to background color
		lamp[i].R1 = arch[0].R1>>5;
		lamp[i].G1 = arch[0].G1>>5;
		lamp[i].B1 = arch[0].B1>>5;
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    for(i=1; i<4; i++)
		{
		arch[i].ltime1+=1;
		if (arch[i].ltime1 >= 4)
			{
			arch[i].ltime1 = 0;
			arch[i].begin -= 3;
			if (arch[i].begin < 0 )
				{
				arch[i].begin = PIXELSARCH/2 -3;
				arch[i].archnum = random(NUMARCHS*2);
				}
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, 6);
		}
    break;

	case ARCHPSYCHO: // 2 lights with RANDOM colors and sustain
	if (newFA == TRUE)
		{
		for(i=1; i<3; i++)
			{
			arch[i].ltime1 = 0; // time accumulator
			arch[i].begin = -2; // which lights to turn on
			arch[i].end = 2;
			arch[i].archnum = random(NUMARCHS); // which arch
			arch[i].ltime1 = 0; // time accumulator
			}
		iprintf("ARCHPSYCHO\r\n");
		newFA = FALSE;
		}
    for(i=1; i<3; i++)
		{

		arch[i].ltime1 = 0;
		arch[i].begin += 2;

		if (arch[i].begin >= PIXELSARCH)
				{
				arch[i].begin = -2;
				arch[i].end = 2;
				arch[i].archnum = random(NUMARCHS*2);
				}


		tempINT162 = random(1323)-300;
		if (tempINT162<0)
				tempINT162 = 0;
		arch[i].R1 = tempINT162;
		tempINT162 = random(1323)-300;
		if (tempINT162<0)
				tempINT162 = 0;
		arch[i].G1 = tempINT162;
		tempINT162 = random(1323)-300;
		if (tempINT162<0)
				tempINT162 = 0;
		arch[i].B1 = tempINT162;
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1,2);
		}
    for (i = start; i<start+num; i++)
		{
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    break;

	case ARCHFIREFLY2: // 2 WHITE lights with background color VER
	if (newFA == TRUE)
		{
		i=0; // ARCH 0 IS ALWAYS THE BACKGROUND COLOR
		arch[i].R1 = 0;
		arch[i].G1 = 0;
		arch[i].B1 = 0;
		arch[i].ltime1 = 0; // time accumulator
		for(i=1; i<3; i++)
			{
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
				tempINT161 = 0;
			arch[i].R1 = tempINT161;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
				tempINT161 = 0;
			arch[i].G1 = tempINT161;
			tempINT161 = random(1223)-200;
			if (tempINT161 <0)
				tempINT161 = 0;
			arch[i].B1 = tempINT161;
			arch[i].ltime1 = 0; // time accumulator
			arch[i].begin = -2; // which lights to turn on
			arch[i].end = 0;
			arch[i].archnum = random(NUMARCHS*2); // which arch
			arch[i].ltime1 = 0; // time accumulator
			}
		pixcount = random(5)+3;
		iprintf("ARCHFIREFLY2\r\n");
		newFA = FALSE;
		}
    for (i = start; i<start+num; i++)
		{
 // set lights to background color
		lamp[i].R1 = arch[0].R1;
		lamp[i].G1 = arch[0].G1;
		lamp[i].B1 = arch[0].B1;
		}
    for(i=1; i<3; i++)
		{
		arch[i].begin += pixcount;
		if (arch[i].begin >= PIXELSARCH)
			{
			arch[i].begin = -pixcount;
			arch[i].archnum = random(NUMARCHS*2);
			}
		setarch(start, arch[i].archnum, arch[i].begin, arch[i].R1, arch[i].G1, arch[i].B1, pixcount);
		}// for
    for (i = start; i<start+num; i++)
		{
		setlight(i,lamp[i].R1, lamp[i].G1, lamp[i].B1);
		}
    break;

	case TREE: // 400 lights read from an SDCard
	if (newFA == TRUE)
		{
		iprintf("TREE-");
		RandomTree(0);
		}
	else
		{
		OutputTree(start);
		}
	break;

	}
}
